"""Simple Graphics

Module designed for use for introductory computer programming.

Wrapper around standard Tk module to enable old-school monochrome graphics.
Paint/unpaint "cells" on screen.

author: Michael Siff
year: 2022

"""

import tkinter as Tk
import warnings

ROWS = 25
COLUMNS = 40
SCALE = 20
BGCOLOR = 'black'
MONOCOLOR = 'green'
PAD = 3
TITLE = 'simple graphics'


# -----------------------------------------------------------------


def open_window():
    """
    Opens simple graphics window that represents a COLUMNSxROWS grid of "cells"
    where each cell is SCALExSCALE pixels.
    """
    global _g
    if _g is None:
        _g = _Graphics()
        _g.root = Tk.Tk()
        _g.r = {}   # pos->id already drawn rectangles
        _g.root.title(TITLE)
        _g.root.wm_protocol('WM_DELETE_WINDOW', _close_window)
        frame = Tk.Frame(_g.root)
        frame.pack()
        _g.canvas = Tk.Canvas(frame,
                              width=COLUMNS*SCALE + PAD,
                              height=ROWS*SCALE + PAD,
                              bg=BGCOLOR)
        _g.canvas.pack()
        _g.root.update()
    else:
        warnings.warn('window already open')


def paint(x, y):
    _verify()
    if (x, y) not in _g.r:
        l = x*SCALE + PAD
        t = y*SCALE + PAD
        r = l + SCALE
        b = t + SCALE
        _g.r[(x, y)] = _g.canvas.create_rectangle(l, t, r, b, fill=MONOCOLOR)
        _g.root.update()


def unpaint(x, y):
    _verify()
    p = (x, y)
    if p in _g.r:
        _g.canvas.delete(_g.r[p])
        del _g.r[p]
        _g.root.update()


def clear():
    """Clears graphics window. (All cells reset to background color.)"""
    _verify()
    _g.r.clear()
    _g.canvas.delete('all')
    _g.root.update()


def _close_window():
    """
    Effectively closes graphics window.
    Limitations of Tk makes this not necessarily actually close the window,
    but next call to open_window will close it before opening new window.
    """
    global _g
    if _g:
        _g.root.quit()
        _g.root.destroy()
        _g = None
    else:
        warnings.warn('no active window currently open')

# ----------------------


class GraphicsException(Exception):
    pass


def _verify():
    if not (_g and _g.root):
        raise GraphicsException('no active graphics window')


class _Graphics:
    pass


_g = None


# -------


if __name__ == '__main__':
    open_window()
    x_base = 0
    for y in range(ROWS):
        for x in range(x_base, COLUMNS, 2):
            paint(x, y)
        x_base = 1 - x_base
    # time.sleep(5)
    x_base = 0
    for y in range(ROWS):
        for x in range(x_base, COLUMNS, 2):
            unpaint(x, y)
        x_base = 1 - x_base
